
From: Ben Slusky <sluskyb@paranoiacs.org>

We need to set the hardsect_size of the loop device to that of the real
device.

The loop device advertises a block size of 1024 even when configured over a
cdrom.

When burning a ext2 on a cd, and mounting it directly, I get:

	blocksize=2048;

when I losetup /dev/loop0 /dev/cdrom, and then try to mount, I get:

blocksize=1024; and then misaligned transfer; this results in not being able
to read the superblock.

The loop device should be changed to export the same blocksize of the
underlying device



---

 drivers/block/loop.c |    7 +++++--
 1 files changed, 5 insertions(+), 2 deletions(-)

diff -puN drivers/block/loop.c~loop-fix-hardsect drivers/block/loop.c
--- 25/drivers/block/loop.c~loop-fix-hardsect	2004-01-08 21:20:37.000000000 -0800
+++ 25-akpm/drivers/block/loop.c	2004-01-08 21:22:46.000000000 -0800
@@ -732,8 +732,6 @@ static int loop_set_fd(struct loop_devic
 	mapping_set_gfp_mask(inode->i_mapping,
 			     lo->old_gfp_mask & ~(__GFP_IO|__GFP_FS));
 
-	set_blocksize(bdev, lo_blocksize);
-
 	lo->lo_bio = lo->lo_biotail = NULL;
 
 	/*
@@ -752,11 +750,16 @@ static int loop_set_fd(struct loop_devic
 		blk_queue_max_sectors(lo->lo_queue, q->max_sectors);
 		blk_queue_max_phys_segments(lo->lo_queue,q->max_phys_segments);
 		blk_queue_max_hw_segments(lo->lo_queue, q->max_hw_segments);
+		blk_queue_hardsect_size(lo->lo_queue, queue_hardsect_size(q));
 		blk_queue_max_segment_size(lo->lo_queue, q->max_segment_size);
 		blk_queue_segment_boundary(lo->lo_queue, q->seg_boundary_mask);
 		blk_queue_merge_bvec(lo->lo_queue, q->merge_bvec_fn);
 	}
 
+	error = set_blocksize(bdev, lo_blocksize);
+	if (error)
+		goto out_putf;
+
 	kernel_thread(loop_thread, lo, CLONE_KERNEL);
 	down(&lo->lo_sem);
 

_
